"""Unit tests for bedrock-supported APIs."""

from datetime import datetime
from unittest import SkipTest

import boto3
import pytest
from botocore.exceptions import ClientError
from freezegun import freeze_time

from moto import mock_aws, settings

DEFAULT_REGION = "us-east-1"

# See our Development Tips on writing tests for hints on how to write good tests:
# http://docs.getmoto.org/en/latest/docs/contributing/development_tips/tests.html


@mock_aws
def test_create_model_customization_job():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    # create a test s3 client and bucket
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")

    resp = client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    assert (
        resp["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob"
    )


@mock_aws
def test_get_model_customization_job():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    resp = client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.get_model_customization_job(jobIdentifier="testjob")

    assert (
        resp["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob"
    )
    assert resp["roleArn"] == "testrole"


@mock_aws
def test_get_model_invocation_logging_configuration():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    logging_config = {
        "cloudWatchConfig": {
            "logGroupName": "Test",
            "roleArn": "testrole",
            "largeDataDeliveryS3Config": {
                "bucketName": "testbucket",
            },
        },
        "s3Config": {
            "bucketName": "testconfigbucket",
        },
    }
    client.put_model_invocation_logging_configuration(loggingConfig=logging_config)
    response = client.get_model_invocation_logging_configuration()
    assert response["loggingConfig"]["cloudWatchConfig"]["logGroupName"] == "Test"
    assert response["loggingConfig"]["s3Config"]["bucketName"] == "testconfigbucket"


@mock_aws
def test_put_model_invocation_logging_configuration():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    logging_config = {
        "cloudWatchConfig": {
            "logGroupName": "Test",
            "roleArn": "testrole",
            "largeDataDeliveryS3Config": {
                "bucketName": "testbucket",
            },
        },
        "s3Config": {
            "bucketName": "testconfigbucket",
        },
    }
    client.put_model_invocation_logging_configuration(loggingConfig=logging_config)
    response = client.get_model_invocation_logging_configuration()
    assert response["loggingConfig"]["cloudWatchConfig"]["logGroupName"] == "Test"


@mock_aws
def test_tag_resource_model_customization_job():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    job_arn = client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.tag_resource(
        resourceARN=job_arn["jobArn"],
        tags=[
            {"key": "testkey", "value": "testvalue"},
            {"key": "testkey2", "value": "testvalue2"},
        ],
    )
    resp = client.list_tags_for_resource(resourceARN=job_arn["jobArn"])
    assert resp["tags"][0]["key"] == "testkey"
    assert resp["tags"][1]["value"] == "testvalue2"


@mock_aws
def test_untag_resource():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    job_arn = client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    client.tag_resource(
        resourceARN=job_arn["jobArn"],
        tags=[
            {"key": "testkey", "value": "testvalue"},
            {"key": "testkey2", "value": "testvalue2"},
        ],
    )
    client.untag_resource(resourceARN=job_arn["jobArn"], tagKeys=["testkey"])
    resp = client.list_tags_for_resource(resourceARN=job_arn["jobArn"])

    assert resp["tags"][0]["key"] == "testkey2"
    assert resp["tags"][0]["value"] == "testvalue2"


@mock_aws
def test_untag_resource_custom_model():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    job_arn = client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )

    client.tag_resource(
        resourceARN=job_arn["jobArn"],
        tags=[
            {"key": "testkey", "value": "testvalue"},
            {"key": "testkey2", "value": "testvalue2"},
        ],
    )
    resp = client.untag_resource(resourceARN=job_arn["jobArn"], tagKeys=["testkey"])
    resp = client.list_tags_for_resource(resourceARN=job_arn["jobArn"])

    assert resp["tags"][0]["key"] == "testkey2"
    assert resp["tags"][0]["value"] == "testvalue2"


@mock_aws
def test_list_tags_for_resource():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    job_arn = client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    client.tag_resource(
        resourceARN=job_arn["jobArn"],
        tags=[
            {"key": "testkey", "value": "testvalue"},
            {"key": "testkey2", "value": "testvalue2"},
        ],
    )
    resp = client.list_tags_for_resource(resourceARN=job_arn["jobArn"])

    assert resp["tags"][0]["key"] == "testkey"
    assert resp["tags"][1]["value"] == "testvalue2"


@mock_aws
def test_get_custom_model():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    job_arn = client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.get_custom_model(modelIdentifier="testmodel")
    assert (
        resp["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel"
    )
    assert resp["jobArn"] == job_arn["jobArn"]


@mock_aws
def test_get_custom_model_arn():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    job_arn = client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )

    resp = client.get_custom_model(modelIdentifier="testmodel")
    model = client.get_custom_model(modelIdentifier=resp["modelArn"])
    assert model["modelName"] == "testmodel"
    assert resp["jobArn"] == job_arn["jobArn"]


@mock_aws
def test_get_custom_model_arn_not_found():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )

    resp = client.get_custom_model(modelIdentifier="testmodel")
    with pytest.raises(ClientError) as ex:
        client.get_custom_model(modelIdentifier=(resp["modelArn"] + "no"))
    assert ex.value.response["Error"]["Code"] == "ResourceNotFoundException"


@mock_aws
def test_list_custom_models():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel1",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    client.create_model_customization_job(
        jobName="testjob2",
        customModelName="testmodel2",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.list_custom_models()
    assert len(resp["modelSummaries"]) == 2
    assert (
        resp["modelSummaries"][0]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel1"
    )
    assert (
        resp["modelSummaries"][1]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel2"
    )


@mock_aws
def test_list_model_customization_jobs():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel1",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    client.create_model_customization_job(
        jobName="testjob2",
        customModelName="testmodel2",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.list_model_customization_jobs()
    assert len(resp["modelCustomizationJobSummaries"]) == 2
    assert (
        resp["modelCustomizationJobSummaries"][0]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob"
    )
    assert (
        resp["modelCustomizationJobSummaries"][1]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob2"
    )


@mock_aws
def test_delete_custom_model():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    client.delete_custom_model(modelIdentifier="testmodel")

    with pytest.raises(ClientError) as ex:
        client.get_custom_model(modelIdentifier="testmodel")
    assert ex.value.response["Error"]["Code"] == "ResourceNotFoundException"


@mock_aws
def test_delete_custom_model_not_found():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )

    with pytest.raises(ClientError) as ex:
        client.delete_custom_model(modelIdentifier="testmodel1")
    assert ex.value.response["Error"]["Code"] == "ResourceNotFoundException"


@mock_aws
def test_stop_model_customization_job():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.stop_model_customization_job(jobIdentifier="testjob")
    resp = client.get_model_customization_job(jobIdentifier="testjob")
    assert resp["status"] == "Stopped"


@mock_aws
def test_delete_model_invocation_logging_configuration():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    logging_config = {
        "cloudWatchConfig": {
            "logGroupName": "Test",
            "roleArn": "testrole",
            "largeDataDeliveryS3Config": {
                "bucketName": "testbucket",
            },
        },
        "s3Config": {
            "bucketName": "testconfigbucket",
        },
    }
    client.put_model_invocation_logging_configuration(loggingConfig=logging_config)
    client.delete_model_invocation_logging_configuration()
    assert client.get_model_invocation_logging_configuration()["loggingConfig"] == {}


@mock_aws
def test_create_model_customization_job_bad_training_data_config():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    # create a test s3 client and bucket
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    with pytest.raises(ClientError) as ex:
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "aws:s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    assert ex.value.response["Error"]["Code"] == "ValidationException"


@mock_aws
def test_create_model_customization_job_bad_validation_data_config():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    # create a test s3 client and bucket
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    with pytest.raises(ClientError) as ex:
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            validationDataConfig={
                "validators": [{"s3Uri": "aws:s3://validation_bucket"}]
            },
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    assert ex.value.response["Error"]["Code"] == "ValidationException"


@mock_aws
def test_create_model_customization_job_bad_output_data_config():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    # create a test s3 client and bucket
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    with pytest.raises(ClientError) as ex:
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            validationDataConfig={"validators": [{"s3Uri": "s3://validation_bucket"}]},
            outputDataConfig={"s3Uri": "aws:s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    assert ex.value.response["Error"]["Code"] == "ValidationException"


@mock_aws
def test_create_model_customization_job_duplicate_job_name():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    # create a test s3 client and bucket
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        validationDataConfig={"validators": [{"s3Uri": "s3://validation_bucket"}]},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with pytest.raises(ClientError) as ex:
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel1",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            validationDataConfig={"validators": [{"s3Uri": "s3://validation_bucket"}]},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    assert ex.value.response["Error"]["Code"] == "ResourceInUseException"


@mock_aws
def test_create_model_customization_job_duplicate_model_name():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    # create a test s3 client and bucket
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    client.create_model_customization_job(
        jobName="testjob1",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        validationDataConfig={"validators": [{"s3Uri": "s3://validation_bucket"}]},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with pytest.raises(ClientError) as ex:
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            validationDataConfig={"validators": [{"s3Uri": "s3://validation_bucket"}]},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    assert ex.value.response["Error"]["Code"] == "ResourceInUseException"


@mock_aws
def test_create_model_customization_job_tags():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    # create a test s3 client and bucket
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    resp = client.create_model_customization_job(
        jobName="testjob1",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        validationDataConfig={"validators": [{"s3Uri": "s3://validation_bucket"}]},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
        jobTags=[{"key": "test", "value": "testvalue"}],
        customModelTags=[{"key": "modeltest", "value": "modeltestvalue"}],
    )
    job_tags = client.list_tags_for_resource(resourceARN=resp["jobArn"])
    model_arn = client.list_custom_models()["modelSummaries"][0]["modelArn"]
    model_tags = client.list_tags_for_resource(resourceARN=model_arn)
    assert job_tags["tags"] == [{"key": "test", "value": "testvalue"}]
    assert model_tags["tags"] == [{"key": "modeltest", "value": "modeltestvalue"}]


@mock_aws
def test_get_model_customization_job_not_found():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with pytest.raises(ClientError) as ex:
        client.get_model_customization_job(jobIdentifier="testjob1")
    assert ex.value.response["Error"]["Code"] == "ResourceNotFoundException"


@mock_aws
def test_stop_model_customization_job_not_found():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with pytest.raises(ClientError) as ex:
        client.stop_model_customization_job(jobIdentifier="testjob1")
    assert ex.value.response["Error"]["Code"] == "ResourceNotFoundException"


@mock_aws
def test_list_model_customization_jobs_max_results():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel1",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    client.create_model_customization_job(
        jobName="testjob2",
        customModelName="testmodel2",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.list_model_customization_jobs(maxResults=1)
    assert len(resp["modelCustomizationJobSummaries"]) == 1
    assert (
        resp["modelCustomizationJobSummaries"][0]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob"
    )


@mock_aws
def test_list_model_customization_jobs_name_contains():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel1",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    client.create_model_customization_job(
        jobName="testjob2",
        customModelName="testmodel2",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.list_model_customization_jobs(nameContains="testjob2")
    assert len(resp["modelCustomizationJobSummaries"]) == 1
    assert (
        resp["modelCustomizationJobSummaries"][0]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob2"
    )


@mock_aws
def test_list_model_customization_jobs_creation_time_before():
    if settings.TEST_SERVER_MODE:
        raise SkipTest("Can't freeze time in ServerMode")
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    with freeze_time("2022-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel1",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    client.create_model_customization_job(
        jobName="testjob2",
        customModelName="testmodel2",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.list_model_customization_jobs(
        creationTimeBefore=datetime(2022, 2, 1, 12, 0, 0)
    )
    assert len(resp["modelCustomizationJobSummaries"]) == 1
    assert (
        resp["modelCustomizationJobSummaries"][0]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob"
    )


@mock_aws
def test_list_model_customization_jobs_creation_time_after():
    if settings.TEST_SERVER_MODE:
        raise SkipTest("Can't freeze time in ServerMode")
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    with freeze_time("2022-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel1",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    client.create_model_customization_job(
        jobName="testjob2",
        customModelName="testmodel2",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.list_model_customization_jobs(
        creationTimeAfter=datetime(2022, 2, 1, 12, 0, 0)
    )
    assert len(resp["modelCustomizationJobSummaries"]) == 1
    assert (
        resp["modelCustomizationJobSummaries"][0]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob2"
    )


@mock_aws
def test_list_model_customization_jobs_status():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel1",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    client.create_model_customization_job(
        jobName="testjob2",
        customModelName="testmodel2",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.list_model_customization_jobs(statusEquals="InProgress")
    assert len(resp["modelCustomizationJobSummaries"]) == 2
    assert (
        resp["modelCustomizationJobSummaries"][0]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob"
    )
    assert (
        resp["modelCustomizationJobSummaries"][1]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob2"
    )


@mock_aws
def test_list_model_customization_jobs_ascending_sort():
    if settings.TEST_SERVER_MODE:
        raise SkipTest("Can't freeze time in ServerMode")
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    with freeze_time("2022-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel1",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    client.create_model_customization_job(
        jobName="testjob3",
        customModelName="testmodel3",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with freeze_time("2023-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob2",
            customModelName="testmodel2",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    resp = client.list_model_customization_jobs(
        sortBy="CreationTime", sortOrder="Ascending"
    )
    assert len(resp["modelCustomizationJobSummaries"]) == 3
    assert (
        resp["modelCustomizationJobSummaries"][0]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob"
    )
    assert (
        resp["modelCustomizationJobSummaries"][1]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob2"
    )
    assert (
        resp["modelCustomizationJobSummaries"][2]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob3"
    )


@mock_aws
def test_list_model_customization_jobs_descending_sort():
    if settings.TEST_SERVER_MODE:
        raise SkipTest("Can't freeze time in ServerMode")
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    with freeze_time("2022-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel1",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    client.create_model_customization_job(
        jobName="testjob3",
        customModelName="testmodel3",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with freeze_time("2023-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob2",
            customModelName="testmodel2",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    resp = client.list_model_customization_jobs(
        sortBy="CreationTime", sortOrder="Descending"
    )
    assert len(resp["modelCustomizationJobSummaries"]) == 3
    assert (
        resp["modelCustomizationJobSummaries"][0]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob3"
    )
    assert (
        resp["modelCustomizationJobSummaries"][1]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob2"
    )
    assert (
        resp["modelCustomizationJobSummaries"][2]["jobArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:model-customization-job/testjob"
    )


@mock_aws
def test_list_model_customization_jobs_bad_sort_order():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    with pytest.raises(ClientError) as ex:
        client.list_model_customization_jobs(
            sortBy="CreationTime", sortOrder="decending"
        )
    assert ex.value.response["Error"]["Code"] == "ValidationException"


@mock_aws
def test_list_model_customization_jobs_bad_sort_by():
    if settings.TEST_SERVER_MODE:
        raise SkipTest("Can't freeze time in ServerMode")
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    with freeze_time("2022-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel1",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    client.create_model_customization_job(
        jobName="testjob3",
        customModelName="testmodel3",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with freeze_time("2023-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob2",
            customModelName="testmodel2",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    with pytest.raises(ClientError) as ex:
        client.list_model_customization_jobs(
            sortBy="Creationime", sortOrder="Descending"
        )
    assert ex.value.response["Error"]["Code"] == "ValidationException"


@mock_aws
def test_get_model_invocation_logging_configuration_empty():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    response = client.get_model_invocation_logging_configuration()
    assert response["loggingConfig"] == {}


@mock_aws
def test_list_custom_models_max_results():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    client.create_model_customization_job(
        jobName="testjob2",
        customModelName="testmodel2",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.list_custom_models(maxResults=1)
    assert len(resp["modelSummaries"]) == 1
    assert (
        resp["modelSummaries"][0]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel"
    )


@mock_aws
def test_list_custom_models_name_contains():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel1",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    client.create_model_customization_job(
        jobName="testjob2",
        customModelName="testmodel2",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.list_custom_models(nameContains="testjob2")
    assert len(resp["modelSummaries"]) == 1
    assert (
        resp["modelSummaries"][0]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel2"
    )


@mock_aws
def test_list_custom_models_creation_time_before():
    if settings.TEST_SERVER_MODE:
        raise SkipTest("Can't freeze time in ServerMode")
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    with freeze_time("2022-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    client.create_model_customization_job(
        jobName="testjob2",
        customModelName="testmodel2",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.list_custom_models(creationTimeBefore=datetime(2022, 2, 1, 12, 0, 0))
    assert len(resp["modelSummaries"]) == 1
    assert (
        resp["modelSummaries"][0]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel"
    )


@mock_aws
def test_list_custom_models_creation_time_after():
    if settings.TEST_SERVER_MODE:
        raise SkipTest("Can't freeze time in ServerMode")
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    with freeze_time("2022-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel1",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    client.create_model_customization_job(
        jobName="testjob2",
        customModelName="testmodel2",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.list_custom_models(creationTimeAfter=datetime(2022, 2, 1, 12, 0, 0))
    assert len(resp["modelSummaries"]) == 1
    assert (
        resp["modelSummaries"][0]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel2"
    )


@mock_aws
def test_list_custom_models_ascending_sort():
    if settings.TEST_SERVER_MODE:
        raise SkipTest("Can't freeze time in ServerMode")
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    with freeze_time("2022-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    client.create_model_customization_job(
        jobName="testjob3",
        customModelName="testmodel3",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with freeze_time("2023-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob2",
            customModelName="testmodel2",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    resp = client.list_custom_models(sortBy="CreationTime", sortOrder="Ascending")
    assert len(resp["modelSummaries"]) == 3
    assert (
        resp["modelSummaries"][0]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel"
    )
    assert (
        resp["modelSummaries"][1]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel2"
    )
    assert (
        resp["modelSummaries"][2]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel3"
    )


@mock_aws
def test_list_custom_models_descending_sort():
    if settings.TEST_SERVER_MODE:
        raise SkipTest("Can't freeze time in ServerMode")
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    with freeze_time("2022-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    client.create_model_customization_job(
        jobName="testjob3",
        customModelName="testmodel3",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with freeze_time("2023-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob2",
            customModelName="testmodel2",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    resp = client.list_custom_models(sortBy="CreationTime", sortOrder="Descending")
    assert len(resp["modelSummaries"]) == 3
    assert (
        resp["modelSummaries"][0]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel3"
    )
    assert (
        resp["modelSummaries"][1]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel2"
    )
    assert (
        resp["modelSummaries"][2]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel"
    )


@mock_aws
def test_list_custom_models_bad_sort_order():
    if settings.TEST_SERVER_MODE:
        raise SkipTest("Can't freeze time in ServerMode")
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    with freeze_time("2022-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel1",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    client.create_model_customization_job(
        jobName="testjob3",
        customModelName="testmodel3",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with freeze_time("2023-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob2",
            customModelName="testmodel2",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    with pytest.raises(ClientError) as ex:
        client.list_custom_models(sortBy="CreationTime", sortOrder="decending")
    assert ex.value.response["Error"]["Code"] == "ValidationException"


@mock_aws
def test_list_custom_models_bad_sort_by():
    if settings.TEST_SERVER_MODE:
        raise SkipTest("Can't freeze time in ServerMode")
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    with freeze_time("2022-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob",
            customModelName="testmodel1",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    client.create_model_customization_job(
        jobName="testjob3",
        customModelName="testmodel3",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with freeze_time("2023-01-01 12:00:00"):
        client.create_model_customization_job(
            jobName="testjob2",
            customModelName="testmodel2",
            roleArn="testrole",
            baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
            trainingDataConfig={"s3Uri": "s3://training_bucket"},
            outputDataConfig={"s3Uri": "s3://output_bucket"},
            hyperParameters={"learning_rate": "0.01"},
        )
    with pytest.raises(ClientError) as ex:
        client.list_custom_models(sortBy="Creationime", sortOrder="Descending")
    assert ex.value.response["Error"]["Code"] == "ValidationException"


@mock_aws
def test_list_custom_models_base_model_arn_equals():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    client.create_model_customization_job(
        jobName="testjob2",
        customModelName="testmodel2",
        roleArn="testrole",
        baseModelIdentifier="amazon.titan-text-lite-v1",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    resp = client.list_custom_models(
        baseModelArnEquals="arn:aws:bedrock:us-east-1::foundation-model/anthropic.claude-3-sonnet-20240229-v1:0"
    )
    assert len(resp["modelSummaries"]) == 1
    assert (
        resp["modelSummaries"][0]["modelArn"]
        == "arn:aws:bedrock:us-east-1:123456789012:custom-model/testmodel"
    )


@mock_aws
def test_tag_resource_not_found():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    job_arn = client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with pytest.raises(ClientError) as ex:
        client.tag_resource(
            resourceARN=job_arn["jobArn"] + "no",
            tags=[
                {"key": "testkey", "value": "testvalue"},
                {"key": "testkey2", "value": "testvalue2"},
            ],
        )
    assert ex.value.response["Error"]["Code"] == "ResourceNotFoundException"


@mock_aws
def test_tag_resource_too_many():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    job_arn = client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    with pytest.raises(ClientError) as ex:
        client.tag_resource(
            resourceARN=job_arn["jobArn"],
            tags=[{"key": f"testkey{i}", "value": f"testvalue{i}"} for i in range(51)],
        )
    assert ex.value.response["Error"]["Code"] == "TooManyTagsException"


@mock_aws
def test_untag_resource_not_found():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    job_arn = client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )

    client.tag_resource(
        resourceARN=job_arn["jobArn"],
        tags=[
            {"key": "testkey", "value": "testvalue"},
            {"key": "testkey2", "value": "testvalue2"},
        ],
    )
    with pytest.raises(ClientError) as ex:
        client.untag_resource(resourceARN=job_arn["jobArn"] + "no", tagKeys=["testkey"])
    assert ex.value.response["Error"]["Code"] == "ResourceNotFoundException"


@mock_aws
def test_list_tags_for_resource_not_found():
    client = boto3.client("bedrock", region_name=DEFAULT_REGION)
    s3_client = boto3.client("s3", region_name=DEFAULT_REGION)
    s3_client.create_bucket(Bucket="training_bucket")
    s3_client.create_bucket(Bucket="output_bucket")
    job_arn = client.create_model_customization_job(
        jobName="testjob",
        customModelName="testmodel",
        roleArn="testrole",
        baseModelIdentifier="anthropic.claude-3-sonnet-20240229-v1:0",
        trainingDataConfig={"s3Uri": "s3://training_bucket"},
        outputDataConfig={"s3Uri": "s3://output_bucket"},
        hyperParameters={"learning_rate": "0.01"},
    )
    client.tag_resource(
        resourceARN=job_arn["jobArn"],
        tags=[
            {"key": "testkey", "value": "testvalue"},
            {"key": "testkey2", "value": "testvalue2"},
        ],
    )
    with pytest.raises(ClientError) as ex:
        client.list_tags_for_resource(resourceARN=job_arn["jobArn"] + "no")
    assert ex.value.response["Error"]["Code"] == "ResourceNotFoundException"



import boto3

from moto import mock_aws


@mock_aws
def test_enable_organization_admin_account():
    client = boto3.client("guardduty", region_name="us-east-1")
    resp = client.enable_organization_admin_account(AdminAccountId="")
    assert resp["ResponseMetadata"]["HTTPStatusCode"] == 200


@mock_aws
def test_list_organization_admin_accounts():
    client = boto3.client("guardduty", region_name="us-east-1")
    client.enable_organization_admin_account(AdminAccountId="someaccount")

    resp = client.list_organization_admin_accounts()
    assert len(resp["AdminAccounts"]) == 1
    assert {"AdminAccountId": "someaccount", "AdminStatus": "ENABLED"} in resp[
        "AdminAccounts"
    ]



import re
import string
from typing import Pattern, Union

from moto.moto_api._internal import mock_random as random

MASTER_ACCOUNT_EMAIL = "master@example.com"
DEFAULT_POLICY_ID = "p-FullAWSAccess"
ORGANIZATION_ARN_FORMAT = "arn:{0}:organizations::{1}:organization/{2}"
MASTER_ACCOUNT_ARN_FORMAT = "arn:{0}:organizations::{1}:account/{2}/{1}"
ACCOUNT_ARN_FORMAT = "arn:{0}:organizations::{1}:account/{2}/{3}"
ROOT_ARN_FORMAT = "arn:{0}:organizations::{1}:root/{2}/{3}"
OU_ARN_FORMAT = "arn:{0}:organizations::{1}:ou/{2}/{3}"
SCP_ARN_FORMAT = "arn:{0}:organizations::{1}:policy/{2}/service_control_policy/{3}"
AI_POLICY_ARN_FORMAT = (
    "arn:{0}:organizations::{1}:policy/{2}/aiservices_opt_out_policy/{3}"
)

CHARSET = string.ascii_lowercase + string.digits
ORG_ID_SIZE = 10
ROOT_ID_SIZE = 4
ACCOUNT_ID_SIZE = 12
OU_ID_SUFFIX_SIZE = 8
CREATE_ACCOUNT_STATUS_ID_SIZE = 8
POLICY_ID_SIZE = 8

EMAIL_REGEX = "^.+@[a-zA-Z0-9-.]+.[a-zA-Z]{2,3}|[0-9]{1,3}$"
ORG_ID_REGEX = rf"o-[a-z0-9]{{{ORG_ID_SIZE}}}"
ROOT_ID_REGEX = rf"r-[a-z0-9]{{{ROOT_ID_SIZE}}}"
OU_ID_REGEX = rf"ou-[a-z0-9]{{{ROOT_ID_SIZE}}}-[a-z0-9]{{{OU_ID_SUFFIX_SIZE}}}"
ACCOUNT_ID_REGEX = rf"[0-9]{{{ACCOUNT_ID_SIZE}}}"
CREATE_ACCOUNT_STATUS_ID_REGEX = rf"car-[a-z0-9]{{{CREATE_ACCOUNT_STATUS_ID_SIZE}}}"
POLICY_ID_REGEX = rf"{DEFAULT_POLICY_ID}|p-[a-z0-9]{{{POLICY_ID_SIZE}}}"

PAGINATION_MODEL = {
    "list_accounts": {
        "input_token": "next_token",
        "limit_key": "max_results",
        "limit_default": 100,
        "result_key": "Accounts",
        "unique_attribute": "JoinedTimestamp",
    },
    "list_accounts_for_parent": {
        "input_token": "next_token",
        "limit_key": "max_results",
        "limit_default": 20,
        "result_key": "Accounts",
        "unique_attribute": "JoinedTimestamp",
    },
    "list_organizational_units_for_parent": {
        "input_token": "next_token",
        "limit_key": "max_results",
        "limit_default": 20,
        "result_key": "OrganizationalUnits",
        "unique_attribute": "Id",
    },
}


def make_random_org_id() -> str:
    # The regex pattern for an organization ID string requires "o-"
    # followed by from 10 to 32 lower-case letters or digits.
    # e.g. 'o-vipjnq5z86'
    return "o-" + "".join(random.choice(CHARSET) for x in range(ORG_ID_SIZE))


def make_random_root_id() -> str:
    # The regex pattern for a root ID string requires "r-" followed by
    # from 4 to 32 lower-case letters or digits.
    # e.g. 'r-3zwx'
    return "r-" + "".join(random.choice(CHARSET) for x in range(ROOT_ID_SIZE))


def make_random_ou_id(root_id: str) -> str:
    # The regex pattern for an organizational unit ID string requires "ou-"
    # followed by from 4 to 32 lower-case letters or digits (the ID of the root
    # that contains the OU) followed by a second "-" dash and from 8 to 32
    # additional lower-case letters or digits.
    # e.g. ou-g8sd-5oe3bjaw
    return "-".join(
        [
            "ou",
            root_id.partition("-")[2],
            "".join(random.choice(CHARSET) for x in range(OU_ID_SUFFIX_SIZE)),
        ]
    )


def make_random_account_id() -> str:
    # The regex pattern for an account ID string requires exactly 12 digits.
    # e.g. '488633172133'
    return "".join([random.choice(string.digits) for n in range(ACCOUNT_ID_SIZE)])


def make_random_create_account_status_id() -> str:
    # The regex pattern for an create account request ID string requires
    # "car-" followed by from 8 to 32 lower-case letters or digits.
    # e.g. 'car-35gxzwrp'
    return "car-" + "".join(
        random.choice(CHARSET) for x in range(CREATE_ACCOUNT_STATUS_ID_SIZE)
    )


def make_random_policy_id() -> str:
    # The regex pattern for a policy ID string requires "p-" followed by
    # from 8 to 128 lower-case letters or digits.
    # e.g. 'p-k2av4a8a'
    return "p-" + "".join(random.choice(CHARSET) for x in range(POLICY_ID_SIZE))


def fullmatch(regex: Union[Pattern[str], str], s: str, flags: int = 0) -> bool:
    """Emulate python-3.4 re.fullmatch()."""
    m = re.match(regex, s, flags=flags)
    if m and m.span()[1] == len(s):
        return True
    return False



import datetime
import re
from base64 import b64decode
from typing import Any, List, Optional, Tuple

import xmltodict

from moto.core.base_backend import BackendDict, BaseBackend
from moto.core.common_models import BaseModel
from moto.core.utils import iso_8601_datetime_with_milliseconds, utcnow
from moto.iam.models import AccessKey, iam_backends
from moto.sts.utils import (
    DEFAULT_STS_SESSION_DURATION,
    random_assumed_role_id,
    random_session_token,
)
from moto.utilities.utils import ARN_PARTITION_REGEX, PARTITION_NAMES, get_partition


class Token(BaseModel):
    def __init__(self, duration: int, name: Optional[str] = None):
        now = utcnow()
        self.expiration = now + datetime.timedelta(seconds=duration)
        self.name = name
        self.policy = None

    @property
    def expiration_ISO8601(self) -> str:
        return iso_8601_datetime_with_milliseconds(self.expiration)


class AssumedRole(BaseModel):
    def __init__(
        self,
        account_id: str,
        region_name: str,
        access_key: AccessKey,
        role_session_name: str,
        role_arn: str,
        policy: str,
        duration: int,
        external_id: str,
    ):
        self.account_id = account_id
        self.region_name = region_name
        self.session_name = role_session_name
        self.role_arn = role_arn
        self.policy = policy
        now = utcnow()
        self.expiration = now + datetime.timedelta(seconds=duration)
        self.external_id = external_id
        self.access_key = access_key
        self.access_key_id = access_key.access_key_id
        self.secret_access_key = access_key.secret_access_key
        self.session_token = random_session_token()
        self.partition = get_partition(region_name)

    @property
    def expiration_ISO8601(self) -> str:
        return iso_8601_datetime_with_milliseconds(self.expiration)

    @property
    def user_id(self) -> str:
        iam_backend = iam_backends[self.account_id][self.partition]
        try:
            role_id = iam_backend.get_role_by_arn(arn=self.role_arn).id
        except Exception:
            role_id = "AROA" + random_assumed_role_id()
        return role_id + ":" + self.session_name

    @property
    def arn(self) -> str:
        partition = get_partition(self.region_name)
        return f"arn:{partition}:sts::{self.account_id}:assumed-role/{self.role_arn.split('/')[-1]}/{self.session_name}"


class STSBackend(BaseBackend):
    def __init__(self, region_name: str, account_id: str):
        super().__init__(region_name, account_id)
        self.assumed_roles: List[AssumedRole] = []

    def get_session_token(self, duration: int) -> Token:
        return Token(duration=duration)

    def get_federation_token(self, name: Optional[str], duration: int) -> Token:
        return Token(duration=duration, name=name)

    def assume_role(
        self,
        region_name: str,
        role_session_name: str,
        role_arn: str,
        policy: str,
        duration: int,
        external_id: str,
    ) -> AssumedRole:
        """
        Assume an IAM Role. Note that the role does not need to exist. The ARN can point to another account, providing an opportunity to switch accounts.
        """
        account_id, access_key = self._create_access_key(role=role_arn)
        role = AssumedRole(
            account_id=account_id,
            region_name=region_name,
            access_key=access_key,
            role_session_name=role_session_name,
            role_arn=role_arn,
            policy=policy,
            duration=duration,
            external_id=external_id,
        )
        access_key.role_arn = role_arn
        account_backend = sts_backends[account_id][get_partition(region_name)]
        account_backend.assumed_roles.append(role)
        return role

    def get_assumed_role_from_access_key(
        self, access_key_id: str
    ) -> Optional[AssumedRole]:
        for assumed_role in self.assumed_roles:
            if assumed_role.access_key_id == access_key_id:
                return assumed_role
        return None

    def assume_role_with_web_identity(self, **kwargs: Any) -> AssumedRole:
        return self.assume_role(**kwargs)

    def assume_role_with_saml(self, **kwargs: Any) -> AssumedRole:
        del kwargs["principal_arn"]
        saml_assertion_encoded = kwargs.pop("saml_assertion")
        saml_assertion_decoded = b64decode(saml_assertion_encoded)

        namespaces = {
            "urn:oasis:names:tc:SAML:2.0:protocol": "samlp",
            "urn:oasis:names:tc:SAML:2.0:assertion": "saml",
        }
        saml_assertion = xmltodict.parse(
            saml_assertion_decoded.decode("utf-8"),
            force_cdata=True,
            process_namespaces=True,
            namespaces=namespaces,
            namespace_separator="|",
        )

        target_role = None
        saml_assertion_attributes = saml_assertion["samlp|Response"]["saml|Assertion"][
            "saml|AttributeStatement"
        ]["saml|Attribute"]
        for attribute in saml_assertion_attributes:
            if (
                attribute["@Name"]
                == "https://aws.amazon.com/SAML/Attributes/RoleSessionName"
            ):
                kwargs["role_session_name"] = attribute["saml|AttributeValue"]["#text"]
            if (
                attribute["@Name"]
                == "https://aws.amazon.com/SAML/Attributes/SessionDuration"
            ):
                kwargs["duration"] = int(attribute["saml|AttributeValue"]["#text"])
            if attribute["@Name"] == "https://aws.amazon.com/SAML/Attributes/Role":
                target_role = attribute["saml|AttributeValue"]["#text"].split(",")[0]

        if "duration" not in kwargs:
            kwargs["duration"] = DEFAULT_STS_SESSION_DURATION

        account_id, access_key = self._create_access_key(role=target_role)  # type: ignore
        kwargs["account_id"] = account_id
        kwargs["region_name"] = self.region_name
        kwargs["access_key"] = access_key

        kwargs["external_id"] = None
        kwargs["policy"] = None
        role = AssumedRole(**kwargs)
        self.assumed_roles.append(role)
        return role

    def get_caller_identity(
        self, access_key_id: str, region: str
    ) -> Tuple[str, str, str]:
        assumed_role = self.get_assumed_role_from_access_key(access_key_id)
        if assumed_role:
            return assumed_role.user_id, assumed_role.arn, assumed_role.account_id

        iam_backend = iam_backends[self.account_id][self.partition]
        user = iam_backend.get_user_from_access_key_id(access_key_id)
        if user:
            return user.id, user.arn, user.account_id

        # Default values in case the request does not use valid credentials generated by moto
        partition = get_partition(region)
        user_id = "AKIAIOSFODNN7EXAMPLE"
        arn = f"arn:{partition}:sts::{self.account_id}:user/moto"
        return user_id, arn, self.account_id

    def _create_access_key(self, role: str) -> Tuple[str, AccessKey]:
        account_id_match = re.search(ARN_PARTITION_REGEX + r":iam::([0-9]+).+", role)
        if account_id_match:
            account_id = account_id_match.group(2)
        else:
            account_id = self.account_id
        iam_backend = iam_backends[account_id][self.partition]
        return account_id, iam_backend.create_temp_access_key()


sts_backends = BackendDict(
    STSBackend, "sts", use_boto3_regions=False, additional_regions=PARTITION_NAMES
)



from typing import TYPE_CHECKING, List, TypedDict

if TYPE_CHECKING:
    from typing_extensions import Any, Dict, Required, Union


# NOTE: Typing is based on the following document https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-event-patterns.html
EventMessageType = TypedDict(
    "EventMessageType",
    {
        "version": str,
        "id": str,
        "detail-type": "Required[Union[str, List[str]]]",
        "source": "Required[Union[str, List[str]]]",
        "account": str,
        # support float type for internal use of moto.
        "time": "Union[str, float]",
        "replay-name": str,
        "region": str,
        "resources": List[str],
        "detail": "Required[Dict[str, Any]]",
    },
    total=False,
)

PAGINATION_MODEL = {
    "list_rules": {
        "input_token": "next_token",
        "limit_key": "limit",
        "limit_default": 50,
        "unique_attribute": "arn",
        "fail_on_invalid_token": False,
    },
    "list_rule_names_by_target": {
        "input_token": "next_token",
        "limit_key": "limit",
        "limit_default": 50,
        "unique_attribute": "arn",
        "fail_on_invalid_token": False,
    },
}

_BASE_EVENT_MESSAGE: EventMessageType = {
    "version": "0",
    "id": "17793124-05d4-b198-2fde-7ededc63b103",
    "detail-type": "",
    "source": "",
    "account": "",
    "time": "",
    "region": "",
    "resources": [],
    "detail": {},
}



"""Handles incoming bedrock requests, invokes methods, returns responses."""

import json
from urllib.parse import unquote

from moto.core.responses import BaseResponse

from .models import BedrockBackend, bedrock_backends


class BedrockResponse(BaseResponse):
    """Handler for Bedrock requests and responses."""

    def __init__(self) -> None:
        super().__init__(service_name="bedrock")

    @property
    def bedrock_backend(self) -> BedrockBackend:
        """Return backend instance specific for this region."""
        return bedrock_backends[self.current_account][self.region]

    def create_model_customization_job(self) -> str:
        params = json.loads(self.body)
        job_name = params.get("jobName")
        custom_model_name = params.get("customModelName")
        role_arn = params.get("roleArn")
        client_request_token = params.get("clientRequestToken")
        base_model_identifier = params.get("baseModelIdentifier")
        customization_type = params.get("customizationType")
        custom_model_kms_key_id = params.get("customModelKmsKeyId")
        job_tags = params.get("jobTags")
        custom_model_tags = params.get("customModelTags")
        training_data_config = params.get("trainingDataConfig")
        validation_data_config = params.get("validationDataConfig")
        output_data_config = params.get("outputDataConfig")
        hyper_parameters = params.get("hyperParameters")
        vpc_config = params.get("vpcConfig")
        job_arn = self.bedrock_backend.create_model_customization_job(
            job_name=job_name,
            custom_model_name=custom_model_name,
            role_arn=role_arn,
            client_request_token=client_request_token,
            base_model_identifier=base_model_identifier,
            customization_type=customization_type,
            custom_model_kms_key_id=custom_model_kms_key_id,
            job_tags=job_tags,
            custom_model_tags=custom_model_tags,
            training_data_config=training_data_config,
            validation_data_config=validation_data_config,
            output_data_config=output_data_config,
            hyper_parameters=hyper_parameters,
            vpc_config=vpc_config,
        )
        return json.dumps(dict(jobArn=job_arn))

    def get_model_customization_job(self) -> str:
        job_identifier = self.path.split("/")[-1]
        model_customization_job = self.bedrock_backend.get_model_customization_job(
            job_identifier=job_identifier
        )
        return json.dumps(dict(model_customization_job.to_dict()))

    def get_model_invocation_logging_configuration(self) -> str:
        logging_config = (
            self.bedrock_backend.get_model_invocation_logging_configuration()
        )
        return json.dumps(dict(loggingConfig=logging_config))

    def put_model_invocation_logging_configuration(self) -> None:
        params = json.loads(self.body)
        logging_config = params.get("loggingConfig")
        self.bedrock_backend.put_model_invocation_logging_configuration(
            logging_config=logging_config
        )
        return

    def tag_resource(self) -> None:
        params = json.loads(self.body)
        resource_arn = params.get("resourceARN")
        tags = params.get("tags")
        self.bedrock_backend.tag_resource(
            resource_arn=resource_arn,
            tags=tags,
        )
        return

    def untag_resource(self) -> str:
        params = json.loads(self.body)
        resource_arn = params.get("resourceARN")
        tag_keys = params.get("tagKeys")
        self.bedrock_backend.untag_resource(
            resource_arn=resource_arn,
            tag_keys=tag_keys,
        )
        return json.dumps(dict())

    def list_tags_for_resource(self) -> str:
        params = json.loads(self.body)
        resource_arn = params.get("resourceARN")
        tags = self.bedrock_backend.list_tags_for_resource(
            resource_arn=resource_arn,
        )
        return json.dumps(dict(tags=tags))

    def get_custom_model(self) -> str:
        model_identifier = unquote(self.path.split("/")[-1])
        custom_model = self.bedrock_backend.get_custom_model(
            model_identifier=model_identifier
        )
        return json.dumps(dict(custom_model.to_dict()))

    def list_custom_models(self) -> str:
        params = self._get_params()
        creation_time_before = params.get("creationTimeBefore")
        creation_time_after = params.get("creationTimeAfter")
        name_contains = params.get("nameContains")
        base_model_arn_equals = params.get("baseModelArnEquals")
        foundation_model_arn_equals = params.get("foundationModelArnEquals")
        max_results = params.get("maxResults")
        next_token = params.get("nextToken")
        sort_by = params.get("sortBy")
        sort_order = params.get("sortOrder")

        max_results = int(max_results) if max_results else None
        model_summaries, next_token = self.bedrock_backend.list_custom_models(
            creation_time_before=creation_time_before,
            creation_time_after=creation_time_after,
            name_contains=name_contains,
            base_model_arn_equals=base_model_arn_equals,
            foundation_model_arn_equals=foundation_model_arn_equals,
            max_results=max_results,
            next_token=next_token,
            sort_by=sort_by,
            sort_order=sort_order,
        )
        return json.dumps(dict(nextToken=next_token, modelSummaries=model_summaries))

    def list_model_customization_jobs(self) -> str:
        params = self._get_params()
        creation_time_after = params.get("creationTimeAfter")
        creation_time_before = params.get("creationTimeBefore")
        status_equals = params.get("statusEquals")
        name_contains = params.get("nameContains")
        max_results = params.get("maxResults")
        next_token = params.get("nextToken")
        sort_by = params.get("sortBy")
        sort_order = params.get("sortOrder")

        max_results = int(max_results) if max_results else None
        (
            model_customization_job_summaries,
            next_token,
        ) = self.bedrock_backend.list_model_customization_jobs(
            creation_time_after=creation_time_after,
            creation_time_before=creation_time_before,
            status_equals=status_equals,
            name_contains=name_contains,
            max_results=max_results,
            next_token=next_token,
            sort_by=sort_by,
            sort_order=sort_order,
        )
        return json.dumps(
            dict(
                nextToken=next_token,
                modelCustomizationJobSummaries=model_customization_job_summaries,
            )
        )

    def delete_custom_model(self) -> str:
        model_identifier = self.path.split("/")[-1]
        self.bedrock_backend.delete_custom_model(
            model_identifier=model_identifier,
        )
        return json.dumps(dict())

    def stop_model_customization_job(self) -> str:
        job_identifier = self.path.split("/")[-2]
        self.bedrock_backend.stop_model_customization_job(
            job_identifier=job_identifier,
        )
        return json.dumps(dict())

    def delete_model_invocation_logging_configuration(self) -> str:
        self.bedrock_backend.delete_model_invocation_logging_configuration()
        return json.dumps(dict())



import datetime
import re
import uuid

import boto3
import pytest
from botocore.exceptions import ClientError

from moto import mock_aws
from moto.core import DEFAULT_ACCOUNT_ID as ACCOUNT_ID

TEST_REGION_NAME = "us-east-1"
TEST_ROLE_ARN = f"arn:aws:iam::{ACCOUNT_ID}:role/FakeRole"
GENERIC_TAGS_PARAM = [
    {"Key": "newkey1", "Value": "newval1"},
    {"Key": "newkey2", "Value": "newval2"},
]
TEST_MODEL_NAME = "MyModel"
TEST_ENDPOINT_NAME = "MyEndpoint"
TEST_ENDPOINT_CONFIG_NAME = "MyEndpointConfig"
TEST_VARIANT_NAME = "MyProductionVariant"
TEST_INSTANCE_TYPE = "ml.t2.medium"
TEST_MEMORY_SIZE = 1024
TEST_CONCURRENCY = 10
TEST_PRODUCTION_VARIANTS = [
    {
        "VariantName": TEST_VARIANT_NAME,
        "ModelName": TEST_MODEL_NAME,
        "InitialInstanceCount": 1,
        "InstanceType": TEST_INSTANCE_TYPE,
    },
]
TEST_SERVERLESS_PRODUCTION_VARIANTS = [
    {
        "VariantName": TEST_VARIANT_NAME,
        "ModelName": TEST_MODEL_NAME,
        "ServerlessConfig": {
            "MemorySizeInMB": TEST_MEMORY_SIZE,
            "MaxConcurrency": TEST_CONCURRENCY,
        },
    },
]


@pytest.fixture(name="sagemaker_client")
def fixture_sagemaker_client():
    with mock_aws():
        yield boto3.client("sagemaker", region_name=TEST_REGION_NAME)


def create_endpoint_config_helper(sagemaker_client, production_variants):
    _create_model(sagemaker_client, TEST_MODEL_NAME)

    resp = sagemaker_client.create_endpoint_config(
        EndpointConfigName=TEST_ENDPOINT_CONFIG_NAME,
        ProductionVariants=production_variants,
    )
    assert re.match(
        rf"^arn:aws:sagemaker:.*:.*:endpoint-config/{TEST_ENDPOINT_CONFIG_NAME}$",
        resp["EndpointConfigArn"],
    )

    resp = sagemaker_client.describe_endpoint_config(
        EndpointConfigName=TEST_ENDPOINT_CONFIG_NAME
    )
    assert re.match(
        rf"^arn:aws:sagemaker:.*:.*:endpoint-config/{TEST_ENDPOINT_CONFIG_NAME}$",
        resp["EndpointConfigArn"],
    )
    assert resp["EndpointConfigName"] == TEST_ENDPOINT_CONFIG_NAME
    assert resp["ProductionVariants"] == production_variants


def test_create_endpoint_config(sagemaker_client):
    with pytest.raises(ClientError) as e:
        sagemaker_client.create_endpoint_config(
            EndpointConfigName=TEST_ENDPOINT_CONFIG_NAME,
            ProductionVariants=TEST_PRODUCTION_VARIANTS,
        )
    assert e.value.response["Error"]["Message"].startswith("Could not find model")

    # Testing instance-based endpoint configuration
    create_endpoint_config_helper(sagemaker_client, TEST_PRODUCTION_VARIANTS)


def test_create_endpoint_config_serverless(sagemaker_client):
    with pytest.raises(ClientError) as e:
        sagemaker_client.create_endpoint_config(
            EndpointConfigName=TEST_ENDPOINT_CONFIG_NAME,
            ProductionVariants=TEST_SERVERLESS_PRODUCTION_VARIANTS,
        )
    assert e.value.response["Error"]["Message"].startswith("Could not find model")

    # Testing serverless endpoint configuration
    create_endpoint_config_helper(sagemaker_client, TEST_SERVERLESS_PRODUCTION_VARIANTS)


def test_delete_endpoint_config(sagemaker_client):
    _create_model(sagemaker_client, TEST_MODEL_NAME)
    resp = sagemaker_client.create_endpoint_config(
        EndpointConfigName=TEST_ENDPOINT_CONFIG_NAME,
        ProductionVariants=TEST_PRODUCTION_VARIANTS,
    )
    assert re.match(
        rf"^arn:aws:sagemaker:.*:.*:endpoint-config/{TEST_ENDPOINT_CONFIG_NAME}$",
        resp["EndpointConfigArn"],
    )

    resp = sagemaker_client.describe_endpoint_config(
        EndpointConfigName=TEST_ENDPOINT_CONFIG_NAME
    )
    assert re.match(
        rf"^arn:aws:sagemaker:.*:.*:endpoint-config/{TEST_ENDPOINT_CONFIG_NAME}$",
        resp["EndpointConfigArn"],
    )

    sagemaker_client.delete_endpoint_config(
        EndpointConfigName=TEST_ENDPOINT_CONFIG_NAME
    )
    with pytest.raises(ClientError) as e:
        sagemaker_client.describe_endpoint_config(
            EndpointConfigName=TEST_ENDPOINT_CONFIG_NAME
        )
    assert e.value.response["Error"]["Message"].startswith(
        "Could not find endpoint configuration"
    )

    with pytest.raises(ClientError) as e:
        sagemaker_client.delete_endpoint_config(
            EndpointConfigName=TEST_ENDPOINT_CONFIG_NAME
        )
    assert e.value.response["Error"]["Message"].startswith(
        "Could not find endpoint configuration"
    )


def test_create_endpoint_invalid_instance_type(sagemaker_client):
    _create_model(sagemaker_client, TEST_MODEL_NAME)

    instance_type = "InvalidInstanceType"
    production_variants = TEST_PRODUCTION_VARIANTS
    production_variants[0]["InstanceType"] = instance_type

    with pytest.raises(ClientError) as e:
        sagemaker_client.create_endpoint_config(
            EndpointConfigName=TEST_ENDPOINT_CONFIG_NAME,
            ProductionVariants=production_variants,
        )
    assert e.value.response["Error"]["Code"] == "ValidationException"
    expected_message = (
        f"Value '{instance_type}' at 'instanceType' failed to satisfy "
        "constraint: Member must satisfy enum value set: ["
    )
    assert expected_message in e.value.response["Error"]["Message"]


def test_create_endpoint_invalid_memory_size(sagemaker_client):
    _create_model(sagemaker_client, TEST_MODEL_NAME)

    memory_size = 1111
    production_variants = TEST_SERVERLESS_PRODUCTION_VARIANTS
    production_variants[0]["ServerlessConfig"]["MemorySizeInMB"] = memory_size

    with pytest.raises(ClientError) as e:
        sagemaker_client.create_endpoint_config(
            EndpointConfigName=TEST_ENDPOINT_CONFIG_NAME,
            ProductionVariants=production_variants,
        )
    assert e.value.response["Error"]["Code"] == "ValidationException"
    expected_message = (
        f"Value '{memory_size}' at 'MemorySizeInMB' failed to satisfy "
        "constraint: Member must satisfy enum value set: ["
    )
    assert expected_message in e.value.response["Error"]["Message"]


def test_create_endpoint(sagemaker_client):
    with pytest.raises(ClientError) as e:
        sagemaker_client.create_endpoint(
            EndpointName=TEST_ENDPOINT_NAME,
            EndpointConfigName="NonexistentEndpointConfig",
        )
    assert e.value.response["Error"]["Message"].startswith(
        "Could not find endpoint configuration"
    )

    _create_model(sagemaker_client, TEST_MODEL_NAME)

    _create_endpoint_config(
        sagemaker_client, TEST_ENDPOINT_CONFIG_NAME, TEST_MODEL_NAME
    )

    resp = sagemaker_client.create_endpoint(
        EndpointName=TEST_ENDPOINT_NAME,
        EndpointConfigName=TEST_ENDPOINT_CONFIG_NAME,
        Tags=GENERIC_TAGS_PARAM,
    )
    assert re.match(
        rf"^arn:aws:sagemaker:.*:.*:endpoint/{TEST_ENDPOINT_NAME}$", resp["EndpointArn"]
    )

    resp = sagemaker_client.describe_endpoint(EndpointName=TEST_ENDPOINT_NAME)
    assert re.match(
        rf"^arn:aws:sagemaker:.*:.*:endpoint/{TEST_ENDPOINT_NAME}$", resp["EndpointArn"]
    )
    assert resp["EndpointName"] == TEST_ENDPOINT_NAME
    assert resp["EndpointConfigName"] == TEST_ENDPOINT_CONFIG_NAME
    assert resp["EndpointStatus"] == "InService"
    assert isinstance(resp["CreationTime"], datetime.datetime)
    assert isinstance(resp["LastModifiedTime"], datetime.datetime)
    assert resp["ProductionVariants"][0]["VariantName"] == TEST_VARIANT_NAME

    resp = sagemaker_client.list_tags(ResourceArn=resp["EndpointArn"])
    assert resp["Tags"] == GENERIC_TAGS_PARAM


def test_delete_endpoint(sagemaker_client):
    _set_up_sagemaker_resources(
        sagemaker_client, TEST_ENDPOINT_NAME, TEST_ENDPOINT_CONFIG_NAME, TEST_MODEL_NAME
    )

    sagemaker_client.delete_endpoint(EndpointName=TEST_ENDPOINT_NAME)
    with pytest.raises(ClientError) as e:
        sagemaker_client.describe_endpoint(EndpointName=TEST_ENDPOINT_NAME)
    assert e.value.response["Error"]["Message"].startswith("Could not find endpoint")

    with pytest.raises(ClientError) as e:
        sagemaker_client.delete_endpoint(EndpointName=TEST_ENDPOINT_NAME)
    assert e.value.response["Error"]["Message"].startswith("Could not find endpoint")


def test_add_tags_endpoint(sagemaker_client):
    _set_up_sagemaker_resources(
        sagemaker_client, TEST_ENDPOINT_NAME, TEST_ENDPOINT_CONFIG_NAME, TEST_MODEL_NAME
    )

    resource_arn = (
        f"arn:aws:sagemaker:{TEST_REGION_NAME}:{ACCOUNT_ID}"
        f":endpoint/{TEST_ENDPOINT_NAME}"
    )
    response = sagemaker_client.add_tags(
        ResourceArn=resource_arn, Tags=GENERIC_TAGS_PARAM
    )
    assert response["ResponseMetadata"]["HTTPStatusCode"] == 200

    response = sagemaker_client.list_tags(ResourceArn=resource_arn)
    assert response["Tags"] == GENERIC_TAGS_PARAM


def test_delete_tags_endpoint(sagemaker_client):
    _set_up_sagemaker_resources(
        sagemaker_client, TEST_ENDPOINT_NAME, TEST_ENDPOINT_CONFIG_NAME, TEST_MODEL_NAME
    )

    resource_arn = (
        f"arn:aws:sagemaker:{TEST_REGION_NAME}:{ACCOUNT_ID}"
        f":endpoint/{TEST_ENDPOINT_NAME}"
    )
    response = sagemaker_client.add_tags(
        ResourceArn=resource_arn, Tags=GENERIC_TAGS_PARAM
    )
    assert response["ResponseMetadata"]["HTTPStatusCode"] == 200

    tag_keys = [tag["Key"] for tag in GENERIC_TAGS_PARAM]
    response = sagemaker_client.delete_tags(ResourceArn=resource_arn, TagKeys=tag_keys)
    assert response["ResponseMetadata"]["HTTPStatusCode"] == 200

    response = sagemaker_client.list_tags(ResourceArn=resource_arn)
    assert response["Tags"] == []


def test_list_tags_endpoint(sagemaker_client):
    _set_up_sagemaker_resources(
        sagemaker_client, TEST_ENDPOINT_NAME, TEST_ENDPOINT_CONFIG_NAME, TEST_MODEL_NAME
    )

    tags = []
    for _ in range(80):
        tags.append({"Key": str(uuid.uuid4()), "Value": "myValue"})

    resource_arn = (
        f"arn:aws:sagemaker:{TEST_REGION_NAME}:{ACCOUNT_ID}"
        f":endpoint/{TEST_ENDPOINT_NAME}"
    )
    response = sagemaker_client.add_tags(ResourceArn=resource_arn, Tags=tags)
    assert response["ResponseMetadata"]["HTTPStatusCode"] == 200

    response = sagemaker_client.list_tags(ResourceArn=resource_arn)
    assert len(response["Tags"]) == 50
    assert response["Tags"] == tags[:50]

    response = sagemaker_client.list_tags(
        ResourceArn=resource_arn, NextToken=response["NextToken"]
    )
    assert len(response["Tags"]) == 30
    assert response["Tags"] == tags[50:]


def test_update_endpoint_weights_and_capacities_one_variant(sagemaker_client):
    _set_up_sagemaker_resources(
        sagemaker_client, TEST_ENDPOINT_NAME, TEST_ENDPOINT_CONFIG_NAME, TEST_MODEL_NAME
    )

    new_desired_weight = 1.5
    new_desired_instance_count = 123

    response = sagemaker_client.update_endpoint_weights_and_capacities(
        EndpointName=TEST_ENDPOINT_NAME,
        DesiredWeightsAndCapacities=[
            {
                "VariantName": TEST_VARIANT_NAME,
                "DesiredWeight": new_desired_weight,
                "DesiredInstanceCount": new_desired_instance_count,
            },
        ],
    )
    assert re.match(
        rf"^arn:aws:sagemaker:.*:.*:endpoint/{TEST_ENDPOINT_NAME}$",
        response["EndpointArn"],
    )

    resp = sagemaker_client.describe_endpoint(EndpointName=TEST_ENDPOINT_NAME)
    assert re.match(
        rf"^arn:aws:sagemaker:.*:.*:endpoint/{TEST_ENDPOINT_NAME}$", resp["EndpointArn"]
    )
    assert resp["EndpointName"] == TEST_ENDPOINT_NAME
    assert resp["EndpointConfigName"] == TEST_ENDPOINT_CONFIG_NAME
    assert resp["EndpointStatus"] == "InService"
    assert isinstance(resp["CreationTime"], datetime.datetime)
    assert isinstance(resp["LastModifiedTime"], datetime.datetime)

    assert resp["ProductionVariants"][0]["VariantName"] == TEST_VARIANT_NAME
    assert (
        resp["ProductionVariants"][0]["DesiredInstanceCount"]
        == new_desired_instance_count
    )
    assert (
        resp["ProductionVariants"][0]["CurrentInstanceCount"]
        == new_desired_instance_count
    )
    assert resp["ProductionVariants"][0]["DesiredWeight"] == new_desired_weight
    assert resp["ProductionVariants"][0]["CurrentWeight"] == new_desired_weight


def test_update_endpoint_weights_and_capacities_two_variants(sagemaker_client):
    production_variants = [
        {
            "VariantName": "MyProductionVariant1",
            "ModelName": TEST_MODEL_NAME,
            "InitialInstanceCount": 1,
            "InstanceType": TEST_INSTANCE_TYPE,
        },
        {
            "VariantName": "MyProductionVariant2",
            "ModelName": TEST_MODEL_NAME,
            "InitialInstanceCount": 1,
            "InstanceType": TEST_INSTANCE_TYPE,
        },
    ]

    _set_up_sagemaker_resources(
        sagemaker_client,
        TEST_ENDPOINT_NAME,
        TEST_ENDPOINT_CONFIG_NAME,
        TEST_MODEL_NAME,
        production_variants,
    )

    desired_weights_and_capacities = [
        {
            "VariantName": "MyProductionVariant1",
            "DesiredWeight": 1.5,
            "DesiredInstanceCount": 123,
        },
        {
            "VariantName": "MyProductionVariant2",
            "DesiredWeight": 1.5,
            "DesiredInstanceCount": 123,
        },
    ]

    new_desired_weight = 1.5
    new_desired_instance_count = 123

    response = sagemaker_client.update_endpoint_weights_and_capacities(
        EndpointName=TEST_ENDPOINT_NAME,
        DesiredWeightsAndCapacities=desired_weights_and_capacities,
    )
    assert re.match(
        rf"^arn:aws:sagemaker:.*:.*:endpoint/{TEST_ENDPOINT_NAME}$",
        response["EndpointArn"],
    )

    resp = sagemaker_client.describe_endpoint(EndpointName=TEST_ENDPOINT_NAME)
    assert re.match(
        rf"^arn:aws:sagemaker:.*:.*:endpoint/{TEST_ENDPOINT_NAME}$", resp["EndpointArn"]
    )
    assert resp["EndpointName"] == TEST_ENDPOINT_NAME
    assert resp["EndpointConfigName"] == TEST_ENDPOINT_CONFIG_NAME
    assert resp["EndpointStatus"] == "InService"
    assert isinstance(resp["CreationTime"], datetime.datetime)
    assert isinstance(resp["LastModifiedTime"], datetime.datetime)

    assert resp["ProductionVariants"][0]["VariantName"] == "MyProductionVariant1"
    assert (
        resp["ProductionVariants"][0]["DesiredInstanceCount"]
        == new_desired_instance_count
    )
    assert (
        resp["ProductionVariants"][0]["CurrentInstanceCount"]
        == new_desired_instance_count
    )
    assert resp["ProductionVariants"][0]["DesiredWeight"] == new_desired_weight
    assert resp["ProductionVariants"][0]["CurrentWeight"] == new_desired_weight

    assert resp["ProductionVariants"][1]["VariantName"] == "MyProductionVariant2"
    assert (
        resp["ProductionVariants"][1]["DesiredInstanceCount"]
        == new_desired_instance_count
    )
    assert (
        resp["ProductionVariants"][1]["CurrentInstanceCount"]
        == new_desired_instance_count
    )
    assert resp["ProductionVariants"][1]["DesiredWeight"] == new_desired_weight
    assert resp["ProductionVariants"][1]["CurrentWeight"] == new_desired_weight


def test_update_endpoint_weights_and_capacities_should_throw_clienterror_no_variant(
    sagemaker_client,
):
    _set_up_sagemaker_resources(
        sagemaker_client, TEST_ENDPOINT_NAME, TEST_ENDPOINT_CONFIG_NAME, TEST_MODEL_NAME
    )

    old_resp = sagemaker_client.describe_endpoint(EndpointName=TEST_ENDPOINT_NAME)
    del old_resp["ResponseMetadata"]

    variant_name = "SillyNotCorrectName"
    new_desired_weight = 1.5
    new_desired_instance_count = 123

    with pytest.raises(ClientError) as exc:
        sagemaker_client.update_endpoint_weights_and_capacities(
            EndpointName=TEST_ENDPOINT_NAME,
            DesiredWeightsAndCapacities=[
                {
                    "VariantName": variant_name,
                    "DesiredWeight": new_desired_weight,
                    "DesiredInstanceCount": new_desired_instance_count,
                },
            ],
        )

    err = exc.value.response["Error"]
    assert err["Message"] == (
        f'The variant name(s) "{variant_name}" is/are not present within '
        f'endpoint configuration "{TEST_ENDPOINT_CONFIG_NAME}".'
    )

    resp = sagemaker_client.describe_endpoint(EndpointName=TEST_ENDPOINT_NAME)
    del resp["ResponseMetadata"]
    assert resp == old_resp


def test_update_endpoint_weights_and_capacities_should_throw_clienterror_no_endpoint(
    sagemaker_client,
):
    _set_up_sagemaker_resources(
        sagemaker_client, TEST_ENDPOINT_NAME, TEST_ENDPOINT_CONFIG_NAME, TEST_MODEL_NAME
    )

    old_resp = sagemaker_client.describe_endpoint(EndpointName=TEST_ENDPOINT_NAME)
    del old_resp["ResponseMetadata"]

    endpoint_name = "SillyEndpointName"
    variant_name = "SillyNotCorrectName"
    new_desired_weight = 1.5
    new_desired_instance_count = 123

    with pytest.raises(ClientError) as exc:
        sagemaker_client.update_endpoint_weights_and_capacities(
            EndpointName=endpoint_name,
            DesiredWeightsAndCapacities=[
                {
                    "VariantName": variant_name,
                    "DesiredWeight": new_desired_weight,
                    "DesiredInstanceCount": new_desired_instance_count,
                },
            ],
        )

    err = exc.value.response["Error"]
    assert err["Message"] == (
        f'Could not find endpoint "arn:aws:sagemaker:us-east-1:'
        f'{ACCOUNT_ID}:endpoint/{endpoint_name}".'
    )

    resp = sagemaker_client.describe_endpoint(EndpointName=TEST_ENDPOINT_NAME)
    del resp["ResponseMetadata"]
    assert resp == old_resp


def test_update_endpoint_weights_and_capacities_should_throw_clienterror_nonunique_variant(
    sagemaker_client,
):
    _set_up_sagemaker_resources(
        sagemaker_client, TEST_ENDPOINT_NAME, TEST_ENDPOINT_CONFIG_NAME, TEST_MODEL_NAME
    )

    old_resp = sagemaker_client.describe_endpoint(EndpointName=TEST_ENDPOINT_NAME)
    del old_resp["ResponseMetadata"]

    desired_weights_and_capacities = [
        {
            "VariantName": TEST_VARIANT_NAME,
            "DesiredWeight": 1.5,
            "DesiredInstanceCount": 123,
        },
        {
            "VariantName": TEST_VARIANT_NAME,
            "DesiredWeight": 1.5,
            "DesiredInstanceCount": 123,
        },
    ]

    with pytest.raises(ClientError) as exc:
        sagemaker_client.update_endpoint_weights_and_capacities(
            EndpointName=TEST_ENDPOINT_NAME,
            DesiredWeightsAndCapacities=desired_weights_and_capacities,
        )

    err = exc.value.response["Error"]
    assert err["Message"] == (
        f'The variant name "{TEST_VARIANT_NAME}" was non-unique within the request.'
    )

    resp = sagemaker_client.describe_endpoint(EndpointName=TEST_ENDPOINT_NAME)
    del resp["ResponseMetadata"]
    assert resp == old_resp


def _set_up_sagemaker_resources(
    boto_client,
    endpoint_name,
    endpoint_config_name,
    model_name,
    production_variants=None,
):
    _create_model(boto_client, model_name)
    _create_endpoint_config(
        boto_client, endpoint_config_name, model_name, production_variants
    )
    _create_endpoint(boto_client, endpoint_name, endpoint_config_name)


def _create_model(boto_client, model_name):
    resp = boto_client.create_model(
        ModelName=model_name,
        PrimaryContainer={
            "Image": "382416733822.dkr.ecr.us-east-1.amazonaws.com/factorization-machines:1",
            "ModelDataUrl": "s3://MyBucket/model.tar.gz",
        },
        ExecutionRoleArn=TEST_ROLE_ARN,
    )
    assert resp["ResponseMetadata"]["HTTPStatusCode"] == 200


def _create_endpoint_config(
    boto_client, endpoint_config_name, model_name, production_variants=None
):
    if not production_variants:
        production_variants = [
            {
                "VariantName": TEST_VARIANT_NAME,
                "ModelName": model_name,
                "InitialInstanceCount": 1,
                "InstanceType": TEST_INSTANCE_TYPE,
            },
        ]
    resp = boto_client.create_endpoint_config(
        EndpointConfigName=endpoint_config_name, ProductionVariants=production_variants
    )
    assert re.match(
        rf"^arn:aws:sagemaker:.*:.*:endpoint-config/{endpoint_config_name}$",
        resp["EndpointConfigArn"],
    )


def _create_endpoint(boto_client, endpoint_name, endpoint_config_name):
    resp = boto_client.create_endpoint(
        EndpointName=endpoint_name, EndpointConfigName=endpoint_config_name
    )
    assert re.match(
        rf"^arn:aws:sagemaker:.*:.*:endpoint/{endpoint_name}$", resp["EndpointArn"]
    )



from typing import Any, Dict, Iterable, List, Optional

from moto.core.base_backend import BackendDict, BaseBackend
from moto.core.common_models import BaseModel
from moto.moto_api._internal import mock_random
from moto.utilities.tagging_service import TaggingService
from moto.utilities.utils import get_partition

from .exceptions import (
    AppNotFoundException,
    ConfigurationProfileNotFound,
    ConfigurationVersionNotFound,
)


class HostedConfigurationVersion(BaseModel):
    def __init__(
        self,
        app_id: str,
        config_id: str,
        version: int,
        description: str,
        content: str,
        content_type: str,
        version_label: str,
    ):
        self.app_id = app_id
        self.config_id = config_id
        self.version = version
        self.description = description
        self.content = content
        self.content_type = content_type
        self.version_label = version_label

    def get_headers(self) -> Dict[str, Any]:
        return {
            "application-id": self.app_id,
            "configuration-profile-id": self.config_id,
            "version-number": self.version,
            "description": self.description,
            "content-type": self.content_type,
            "VersionLabel": self.version_label,
        }


class ConfigurationProfile(BaseModel):
    def __init__(
        self,
        application_id: str,
        name: str,
        region: str,
        account_id: str,
        description: str,
        location_uri: str,
        retrieval_role_arn: str,
        validators: List[Dict[str, str]],
        _type: str,
    ):
        self.id = mock_random.get_random_hex(7)
        self.arn = f"arn:{get_partition(region)}:appconfig:{region}:{account_id}:application/{application_id}/configurationprofile/{self.id}"
        self.application_id = application_id
        self.name = name
        self.description = description
        self.location_uri = location_uri
        self.retrieval_role_arn = retrieval_role_arn
        self.validators = validators
        self._type = _type
        self.config_versions: Dict[int, HostedConfigurationVersion] = dict()

    def create_version(
        self,
        app_id: str,
        config_id: str,
        description: str,
        content: str,
        content_type: str,
        version_label: str,
    ) -> HostedConfigurationVersion:
        if self.config_versions:
            version = sorted(self.config_versions.keys())[-1] + 1
        else:
            version = 1
        self.config_versions[version] = HostedConfigurationVersion(
            app_id=app_id,
            config_id=config_id,
            version=version,
            description=description,
            content=content,
            content_type=content_type,
            version_label=version_label,
        )
        return self.config_versions[version]

    def get_version(self, version: int) -> HostedConfigurationVersion:
        if version not in self.config_versions:
            raise ConfigurationVersionNotFound
        return self.config_versions[version]

    def delete_version(self, version: int) -> None:
        self.config_versions.pop(version)

    def to_json(self) -> Dict[str, Any]:
        return {
            "Id": self.id,
            "Name": self.name,
            "ApplicationId": self.application_id,
            "Description": self.description,
            "LocationUri": self.location_uri,
            "RetrievalRoleArn": self.retrieval_role_arn,
            "Validators": self.validators,
            "Type": self._type,
        }


class Application(BaseModel):
    def __init__(
        self, name: str, description: Optional[str], region: str, account_id: str
    ):
        self.id = mock_random.get_random_hex(7)
        self.arn = f"arn:{get_partition(region)}:appconfig:{region}:{account_id}:application/{self.id}"
        self.name = name
        self.description = description

        self.config_profiles: Dict[str, ConfigurationProfile] = dict()

    def to_json(self) -> Dict[str, Any]:
        return {
            "Id": self.id,
            "Name": self.name,
            "Description": self.description,
        }


class AppConfigBackend(BaseBackend):
    """Implementation of AppConfig APIs."""

    def __init__(self, region_name: str, account_id: str):
        super().__init__(region_name, account_id)
        self.applications: Dict[str, Application] = dict()
        self.tagger = TaggingService()

    def create_application(
        self, name: str, description: Optional[str], tags: Dict[str, str]
    ) -> Application:
        app = Application(
            name, description, region=self.region_name, account_id=self.account_id
        )
        self.applications[app.id] = app
        self.tag_resource(app.arn, tags)
        return app

    def delete_application(self, app_id: str) -> None:
        self.applications.pop(app_id, None)

    def get_application(self, app_id: str) -> Application:
        if app_id not in self.applications:
            raise AppNotFoundException
        return self.applications[app_id]

    def update_application(
        self, application_id: str, name: str, description: str
    ) -> Application:
        app = self.get_application(application_id)
        if name is not None:
            app.name = name
        if description is not None:
            app.description = description
        return app

    def create_configuration_profile(
        self,
        application_id: str,
        name: str,
        description: str,
        location_uri: str,
        retrieval_role_arn: str,
        validators: List[Dict[str, str]],
        _type: str,
        tags: Dict[str, str],
    ) -> ConfigurationProfile:
        config_profile = ConfigurationProfile(
            application_id=application_id,
            name=name,
            region=self.region_name,
            account_id=self.account_id,
            description=description,
            location_uri=location_uri,
            retrieval_role_arn=retrieval_role_arn,
            validators=validators,
            _type=_type,
        )
        self.tag_resource(config_profile.arn, tags)
        self.get_application(application_id).config_profiles[config_profile.id] = (
            config_profile
        )
        return config_profile

    def delete_configuration_profile(self, app_id: str, config_profile_id: str) -> None:
        self.get_application(app_id).config_profiles.pop(config_profile_id)

    def get_configuration_profile(
        self, app_id: str, config_profile_id: str
    ) -> ConfigurationProfile:
        app = self.get_application(app_id)
        if config_profile_id not in app.config_profiles:
            raise ConfigurationProfileNotFound
        return app.config_profiles[config_profile_id]

    def update_configuration_profile(
        self,
        application_id: str,
        config_profile_id: str,
        name: str,
        description: str,
        retrieval_role_arn: str,
        validators: List[Dict[str, str]],
    ) -> ConfigurationProfile:
        config_profile = self.get_configuration_profile(
            application_id, config_profile_id
        )
        if name is not None:
            config_profile.name = name
        if description is not None:
            config_profile.description = description
        if retrieval_role_arn is not None:
            config_profile.retrieval_role_arn = retrieval_role_arn
        if validators is not None:
            config_profile.validators = validators
        return config_profile

    def list_configuration_profiles(
        self, app_id: str
    ) -> Iterable[ConfigurationProfile]:
        app = self.get_application(app_id)
        return app.config_profiles.values()

    def create_hosted_configuration_version(
        self,
        app_id: str,
        config_profile_id: str,
        description: str,
        content: str,
        content_type: str,
        version_label: str,
    ) -> HostedConfigurationVersion:
        """
        The LatestVersionNumber-parameter is not yet implemented
        """
        profile = self.get_configuration_profile(app_id, config_profile_id)
        return profile.create_version(
            app_id=app_id,
            config_id=config_profile_id,
            description=description,
            content=content,
            content_type=content_type,
            version_label=version_label,
        )

    def get_hosted_configuration_version(
        self, app_id: str, config_profile_id: str, version: int
    ) -> HostedConfigurationVersion:
        profile = self.get_configuration_profile(
            app_id=app_id, config_profile_id=config_profile_id
        )
        return profile.get_version(version)

    def delete_hosted_configuration_version(
        self, app_id: str, config_profile_id: str, version: int
    ) -> None:
        profile = self.get_configuration_profile(
            app_id=app_id, config_profile_id=config_profile_id
        )
        profile.delete_version(version=version)

    def list_tags_for_resource(self, arn: str) -> Dict[str, str]:
        return self.tagger.get_tag_dict_for_resource(arn)

    def tag_resource(self, arn: str, tags: Dict[str, str]) -> None:
        self.tagger.tag_resource(arn, TaggingService.convert_dict_to_tags_input(tags))

    def untag_resource(self, arn: str, tag_keys: List[str]) -> None:
        self.tagger.untag_resource_using_names(arn, tag_keys)


appconfig_backends = BackendDict(AppConfigBackend, "appconfig")
